
/* 
 * SuperCollider3 source file "BBCuelistProc.sc" 
 * Written by Jens Gulden, jgulden@cs.tu-berlin.de.
 * Licensed under the GNU General Public License (GPL),
 * this software comes with NO WARRANTY.
 */

// --- class BBCuelistProc ----------------------------------------------------
// 
BBCuelistProc : BBCutProc  {

    // --- attributes

    var <>verbose; // type  boolean
    var loopCount = 0; // type  int


    // --- relationships

    var <>cuelist; // 0..1-relation to type  SFCuelist



    // --- new(cuelist) : BBCuelistProc ---------------------------------------
    //      
    *new { arg cuelist; // type SFCuelist         
        ^super.new.cuelist_(cuelist); //.verbose_(verbose); // type  BBCuelistProc
    } // end new        


    // --- chooseblock() : void -----------------------------------------------
    //   
    chooseblock {        
        var cues, length, start, slice, secs, blockstart, totaltime, digits;
        cues = this.cuelist.cues;
        if ( (block >= cues.size) , {
            block = 0;
        });
        if ( ( block == 0 ) , {
            if ( ( (loopCount == 0) || (this.cuelist.loop == true) ) , { // first phrase, or looping is activated
                this.newPhraseAccounting;
                loopCount = loopCount + 1;
            },{ // end playing
                loopCount = -1;
            });
        });
        if ( ( loopCount != -1 ) , { // first phrase, or looping is activated
            slice = cues.at(block);
            start = slice.start.time;
            length = slice.length;
            // set global var 'cuts' inherited from BBCutProc
            blockstart = start.asRel.value;
            blocklength = length.asBeats.value; // 'blocklength' is global var
            cuts = [[blocklength, blocklength, blockstart, 1]];
            bbcutsynth.setoffset(blockstart);
            if ( ( true == this.verbose ), {
                digits = Toolbox.countDigits(cues.size);
                if ( ( ( block==0 ) && ( loopCount==1 ) ), {
                    (" Cue "++Toolbox.repeat(" ", (digits*2)-2)++"  Slice           Time                   Length     Total   ").postln;
                    ("-----"++Toolbox.repeat("-", (digits*2)-2)++"|---------------|----------------------|----------|---------").postln;
                });
                ("[" ++ Toolbox.numberFormat( (block+1), 0, 0, digits, "0") ++ "/" ++ Toolbox.numberFormat(cues.size, 0, 0, digits, "0") ++ "] \"" ++ slice.name ++ "\"" ++ Toolbox.repeat(" ", 14 - slice.name.size)).post; slice.start.time.post; "  - ".post; slice.end.time.post; "   ".post;  slice.end.time.diff(slice.start.time).post;  "   ".post;
                (Toolbox.numberFormat(phrasepos * this.cuelist.masterslice.master.secperbeat, 3, 3, 3) ++ "s").postln; // (copied below)
                //                                slice.masterslice...
            });
        },{ // end: no more blocks
            // set global var 'cuts' inherited from BBCutProc
            //cuts = [ [ 0, 0, 0, 1 ] ];
            if ( ( true == this.verbose ), {
                digits = Toolbox.countDigits(cues.size);
                ("["++Toolbox.repeat("-", (digits*2)+1)++"] \"" ++ cuelist.name ++ "\" ended." ++ Toolbox.blanks(41-cuelist.name.size)).post;
                (Toolbox.numberFormat(phrasepos * this.cuelist.masterslice.master.secperbeat, 3, 3, 3) ++ "s").postln; // (copied from above)
            });
            bbcutsynth.kill; // end BBCut
        });
        this.updateblock; 
        this.endBlockAccounting;        
    } // end chooseblock        


} // end BBCuelistProc
