
/* 
 * SuperCollider3 source file "XMLParser.sc" 
 * Written by Jens Gulden, jgulden@cs.tu-berlin.de.
 * Licensed under the GNU General Public License (GPL),
 * this software comes with NO WARRANTY.
 */

// --- class XMLParser --------------------------------------------------------
// 
XMLParser {

    // --- attributes

    var <>parseTypes; // type  TwoWayDictionary
    var <>parseFunctions; // type  TwoWayDictionary
    var <>defaultParseFunction; // type  Function


    // --- new() : XMLParser --------------------------------------------------
    //     
    *new {        
        ^super.new.init; // type  XMLParser
    } // end new        


    // --- init() : void ------------------------------------------------------
    //   
    init {        
        var att, m, c, coll, nameM, name, tagname, ch;
        
        parseFunctions = TwoWayDictionary.new;
        parseTypes = TwoWayDictionary.new;
        defaultParseFunction = { arg o, xml;
            // use introspection to get access to o's attributes
            // iterate over attributes
            xml.getAttributes.keys.do({ arg attname;
                // is there an equally named settable instance variable (via setter-method)?
                //m = o.class.findMethod(attname++"_");
                if ( Toolbox.hasVar(o, attname) , { //o.class.instVarNames.includes(attname++"_")
                    att = xml.getAttribute(attname);
                    Toolbox.setVar(o, attname, att);
                },{
                    ("XML-parser error: defaultParseFunction cannot read attribute '"++attname++"' into object of class "++o.class.name).die;
                });
            });
            // iterate over child-tags
            c = xml.getFirstChild;
            while ( { c != nil } , {
                tagname = c.getTagName;
                ch = this.parse(c, o); // indirect recursion
                // is there an equally named settable instance variable (via setter-method)?
                //m = o.class.findMethod(tagname++"_");
                if ( Toolbox.hasVar(o, tagname) , {
                    //m.value(ch); // set it
                    Toolbox.setVar(o, tagname, att); // set it
                },{
                    // is there a gettable instance variable with the corresponding plural name (via getter method)?
                    //"$$$ FIND OUT ".post; o.post; (" "++tagname++"s").postln;
                    //m = o.class.findMethod(tagname++"s");
                    //"M: ".post; m.postln;
                    if ( Toolbox.hasVar(o, tagname++"s") , {
                        //coll = m.value;
                        coll = Toolbox.getVar(o, tagname++"s");
                        //nameM = ch.class.findMethod("name");
                        //name = nameM.value;
                        //"COLL ".post; coll.class.name.postln;
                        if ( coll.isKindOf(Collection) , {
                            if ( ( coll.isKindOf(Dictionary) ) , {
                                if ( Toolbox.hasVar(ch, "name") , {
                                    name = Toolbox.getVar(ch, "name");
                                    coll.put(name, ch);
                                },{
                                    ("XML-parser error: defaultParseFunction cannot read tag "++tagname++" into Dictionary, no 'name'-variable in instance of kind '"++ch.class.name++"'").die;
                                });
                            },{
                                coll.add(ch);
                            });
                        },{
                            ("XML-parser error: defaultParseFunction cannot read tag '"++tagname++"' into object of class "++o.class.name++" because variable '"++tagname++"s' is not of kind Collection").die;
                        });
                    },{
                        ("XML-parser error: defaultParseFunction cannot read tag '"++tagname++"' into object of class "++o.class.name).die;
                    });
                });
                c = c.getNextSibling;
            });
        };
    } // end init        


    // --- register(tagname, class, function) : XMLParser ---------------------
    //        
    register { arg tagname, class, function; // types String, Class, Function           
        parseTypes.put(tagname, class);
        parseFunctions.put(tagname, function);
        ^this;
    } // end register        


    // --- parse(xml, parent) : Object ----------------------------------------
    //       
    parse { arg xml, parent; // types DOMNode, Object          
        var tagname, type, o;
        
        if ( xml.isKindOf(DOMDocument) , {
            xml = xml.getDocumentElement;
        });
        tagname = xml.getTagName;
        type = parseTypes.at(tagname);
        if ( type == nil , {
            ("XML-parser error: unknown tag-name '"++tagname++"'").die;
        });
        // create new instance of class
        o = (type.name++".new").interpret; // (is there something like 'type.createNewInstance' ?)
        this.parseObject(o, xml, parent);
        ^o; // type  Object
    } // end parse        


    // --- parseObject(o, xml, parent) : void ---------------------------------
    //       
    parseObject { arg o, xml, parent; // types Object, DOMNode, Object           
        var tagname, func;
        
        tagname = xml.getTagName;
        func = parseFunctions.at(tagname);
        if ( func == nil , {
            func = this.defaultParseFunction;
        });
        ^func.value(o, xml, parent);
    } // end parseObject        


} // end XMLParser
